@@ -77,7 +77,7 @@ static int snd_pcm_jack_close(snd_pcm_ioplug_t *io)
 	return 0;
 }
 
-static int snd_pcm_jack_poll_revents(snd_pcm_ioplug_t *io ATTRIBUTE_UNUSED,
+static int snd_pcm_jack_poll_revents(snd_pcm_ioplug_t *io,
 				     struct pollfd *pfds, unsigned int nfds,
 				     unsigned short *revents)
 {
@@ -87,7 +87,9 @@ static int snd_pcm_jack_poll_revents(snd_pcm_ioplug_t *io ATTRIBUTE_UNUSED,
 
 	read(pfds[0].fd, buf, 1);
 
-	*revents = pfds[0].revents;
+	*revents = pfds[0].revents & ~(POLLIN | POLLOUT);
+	if (pfds[0].revents & POLLIN)
+		*revents |= (io->stream == SND_PCM_STREAM_PLAYBACK) ? POLLOUT : POLLIN;
 	return 0;
 }
 
@@ -307,6 +309,19 @@ static int parse_ports(snd_pcm_jack_t *jack, snd_config_t *conf)
 	return 0;
 }
 
+static int make_nonblock(int fd)
+{
+	int fl;
+
+	if ((fl = fcntl(fd, F_GETFL)) < 0)
+		return fl;
+
+	if (fl & O_NONBLOCK)
+		return 0;
+
+	return fcntl(fd, F_SETFL, fl | O_NONBLOCK);
+}
+
 static int snd_pcm_jack_open(snd_pcm_t **pcmp, const char *name,
 			     snd_config_t *playback_conf,
 			     snd_config_t *capture_conf,
@@ -363,6 +378,9 @@ static int snd_pcm_jack_open(snd_pcm_t **pcmp, const char *name,
 
 	socketpair(AF_LOCAL, SOCK_STREAM, 0, fd);
 	
+	make_nonblock(fd[0]);
+	make_nonblock(fd[1]);
+
 	jack->fd = fd[0];
 
 	jack->io.version = SND_PCM_IOPLUG_VERSION;
